/*
btdlets.c Version 0.1.0  Btree counts deleted index
records in database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* returns total blocks and total count */
/* in database */

#include <bthash.h>

void btdlets(btreefmt *bthdr, int *blocks,
   int *count, int *leaf, int *nonleaf,
   int *active, int *deletes,
   int *indxrcd, int *leafrcd)
   {
   BOOL bResult;
   int eofsw;
   DWORD  rdlen;
   blkfmt *blk;
   DWORD  loc;
   DWORD  msgerr;
   LPVOID lpMsgBuf;
   int rcdnum;
   unsigned char *p;
   unsigned char *rcdptr;
   unsigned char buf[65536 + 16];
   bthdr->reason = 0;
   loc = SetFilePointer(bthdr->hndl,ROOT,
      NULL,FILE_BEGIN);
   if (loc != (DWORD) ROOT)
      {
      msgerr = GetLastError();
      bthdr->reason = SYS_ERR + msgerr;
      FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
         FORMAT_MESSAGE_FROM_SYSTEM |
         FORMAT_MESSAGE_IGNORE_INSERTS,
         NULL,             /* source */
         msgerr,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
         (LPTSTR) &lpMsgBuf,
         0,                /* message length */
         NULL);            /* insertion values */
      fprintf(stderr,"btdlets: %s", lpMsgBuf);
      LocalFree(lpMsgBuf);
      fprintf(stderr,"btdlets: seek error\n");
      return;
      } /* if SetFilePointer err */
   *blocks = *count = eofsw = 0;
   *leaf = *nonleaf = *deletes = *active = 0;
   *indxrcd = *leafrcd = 0;
   blk = (blkfmt *) buf;
   while (!eofsw)
      {
      bResult = ReadFile(bthdr->hndl,buf,
         (DWORD) bthdr->blksz,
         &rdlen,NULL);
      if (!bResult)
	 {
         msgerr = GetLastError();
         bthdr->reason = SYS_ERR + msgerr;
         FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,             /* source */
            msgerr,
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR) &lpMsgBuf,
            0,                /* message length */
            NULL);            /* insertion values */
         fprintf(stderr,"btdlets: %s", lpMsgBuf);
         LocalFree(lpMsgBuf);
         fprintf(stderr,"btdlets: "
            "error reading %s",
            bthdr->dbname);
         return;
	 } /* read err */
      if (rdlen == (DWORD) bthdr->blksz)
	 {
         *count += blk->count;
         (*blocks)++;
	 rcdnum = 0;
	 if (blk->right)
	    {
	    (*nonleaf)++;
	    *indxrcd += blk->count;
	    rcdptr = (unsigned char *)
	       blk + bthdr->blkhdr;
	    while (rcdnum < blk->count)
	       {
	       p = (unsigned char *)
		  rcdptr + bthdr->rcdhdr + bthdr->dletofst;
	       if (*p == DELETED)
		  (*deletes)++;
	       else
		  (*active)++;
	       rcdptr = (unsigned char *)
		  rcdptr + bthdr->rcdnode;
	       rcdnum++;
	       } /* examine delete byte */
	    } /* if index node */
	 else
	    {
	    (*leaf)++;
	    *leafrcd += blk->count;
	    } /* else leaf node */
	 } /* if good read */
      else if (!rdlen)
	 {
	 eofsw = 1;
         break;
	 } /* eof */
      else
	 {
         fprintf(stderr,"btdlets: read error\n");
         fprintf(stderr,"bthdr->blksz %d "
	    "rdlen %d\n",
	    bthdr->blksz, rdlen);
         bthdr->reason = BTREE_ERR + 2501;
         return;
	 } /* read err */
      } /* for each block in db */
   } /* btdlets */
